shinyServer( function( input, output, session ) {
  
  
  
  filedata <- reactive({
    infile <- input$datafile
    if ( is.null( infile )) {
      return( NULL )
    } else if ( !infile$type %in% c( 'text/plain', 'text/csv' )) {
      showModal(modalDialog(
        title = "Inccorect file format",
        "You have uplaoded file in unsuported format. Please choose a file with the .txt or .csv extension"
      ))
      return( NULL )
    } else if( nchar(gsub( '[^\t]', '', readChar(infile$datapath, infile$size ))) == 0 ) {
      showModal(modalDialog(
        title = "Inccorect file data",
        "Please choose a tab-delimited file."
      ))
      return( NULL )
    }
    
    df <- read.table( infile$datapath, header=TRUE, sep="\t", na.strings = c( "", "  ", "NA" ), fill = TRUE )
  })
  
  
  
  output$chooseNrOfGroups <- renderUI({ 
    df <-filedata()
    if (is.null( df )) return( NULL )
    
    tagList(
      checkboxInput( "showfile", "Show file?"),
      radioButtons(  
        "nrGroups", 
        "Number of categories:", 
        choices = list("2" = 2, "3" = 3), 
        selected = 2
      )
    )
  })
  
  output$divionBase <- renderUI({ 
    df <-filedata()
    if ( is.null( df ) || is.null( input$nrGroups ) ) return( NULL )
    
    if ( input$nrGroups == 2 ) {
      anvDivisions <- c(
        "Outcome" = "outcome", 
        "Survival" = "survival", 
        "Adapt" = "adapt"
      )
      selectInput( "divisonBase", "Division based on:", anvDivisions )  
    } else {
      return( NULL )
    }
  })
  
  output$tdivionBase <- renderUI({ 
    df <-filedata()
    if ( is.null( df ) || is.null( input$nrGroups ) ) return( NULL )
    
    if ( input$nrGroups == 3 ) {
      anvDivisions <- c (
        "Rolr" = "rolr",
        "Adapt" = "adapt"
      )
      
      selectInput( "tdivisonBase", "Division based on:", anvDivisions )
      
    } else {
      return( NULL )
    }
  })
  
  output$selectedDivisionsIntructions <- renderUI({
    df <-filedata()
    if ( is.null(df) || input$nrGroups != 2 || is.null(input$divisonBase) ) return( NULL )
    
    if ( input$divisonBase == "survival") {
      survivalOptions <- c(
        "Cutp" = "cutp",
        "Maxstat" = "maxstat"
      )
      selectInput( "cutoffMethod", "Method for cutoff determination:", survivalOptions )
    } else {
      return( NULL )
    }
  })
  output$outcomeDivisionsIntructions <- renderUI({
    df <-filedata()
    if ( is.null(df) || input$nrGroups != 2 || is.null(input$divisonBase) ) return( NULL )
    
    if ( input$divisonBase == "outcome" ) {
      outputOptions <- c(
        "Youden" = "youden",
        "ROC(0,1)" = "roc"
      )
      selectInput( "cutoffoutputMethod", "Method for cutoff determination:", outputOptions )
    } else {
      return( NULL )
    }
  })
  
  output$fileInstructions <- renderUI({ 
    df <-filedata()
    if ( is.null( df ) || is.null( input$divisonBase ) ) return( NULL )
    items=names( df )
    names( items ) = items
    
    
    if ( input$nrGroups == 2 && input$divisonBase == "outcome" )  {
      tagList(
        selectInput( "marker", "Biomarker:", items ),
        selectInput( "event", "Outcome:", items )
      )
    } else {
      tagList(
        selectInput( "marker", "Biomarker:", items ),
        selectInput( "survival", "Survival time:", items ),                  
        selectInput( "event", "Outcome:", items )
      )
    }
  })
  
  
  output$filetable <- renderTable({
    filedata()
  })
  
  observeEvent(input$cutoff, {
    
    df <-filedata()
    if( is.null( df ) ) return()
    
    tryCatch({
      if ( input$nrGroups == 2 && input$divisonBase == "outcome" ) {
        if( input$cutoffoutputMethod == "youden") {
          display.results.outcome.youden( input, output, session, df )
        } else {
          display.results.outcome.roc( input, output, session, df )
        }
      } else if ( input$nrGroups == 2 && input$divisonBase == "survival" ) {
        if( input$cutoffMethod == "cutp" ) {
          display.results.survival.cutp( input, output, session, df )
        } else {
          display.results.survival.maxstat( input, output, session, df )
        }
      } else if ( input$nrGroups == 2 && input$divisonBase == "adapt" ) {
        display.results.adapt( input, output, session, df )
      } else if ( input$nrGroups == 3 && input$tdivisonBase == "rolr" ) {
        display.results.threeGroups( input, output, session, df )
      } else if ( input$nrGroups == 3 && input$tdivisonBase == "adapt" ) {
        display.results.threeAdopt( input, output, session, df ) 
      }
    }, error = function( e ) {
      showModal(modalDialog(
        title = "Corrupted data",
        "Data file contains corrupted data. Please check if choosen columns contains data in correct format."
      ))
    })
    
  })
  
  
  display.results.outcome.youden <- function( input, output, session, df ) {
    biomarker <- df[, input$marker ]
    event <- df[, input$event]
    
    methods <- "Youden"
    
    optimal.cutpoint.Youden <- optimal.cutpoints(X = input$marker, status = input$event, tag.healthy = 0,
                                                 methods = methods, data = df, pop.prev = NULL,
                                                 control = control.cutpoints(), ci.fit = FALSE, conf.level = 0.95, trace = FALSE)
    
    Youdenptable <- summary(optimal.cutpoint.Youden)$p.table$Global
    
    Youdenptable$AUC_CI <- NULL
    
    for(k in methods) {
      colnames(Youdenptable[[k]][[1]]) <- k  
    }
    
    rev.row.names <- attributes(Youdenptable)[[1]]
    rev.col.names <- c("method", "cutoff", "se", "sp", "PPV", "NPV", "DLR.Positive", "DLR.Negative", "FP", "FN")
    rev.Youdenptable <- matrix(ncol=10, nrow=length(methods), dimnames=list(rev.row.names, rev.col.names))
    for(k in 1:length(rev.row.names)) {
      method.name <- rev.row.names[[k]]
      rev.Youdenptable[k, 1] <- method.name
      for(j in 2:10){
        rev.Youdenptable[k, j] <- round(Youdenptable[[method.name]][[1]][[j - 1]], digits = 3)
      }
    }
    cutpoint <- Youdenptable[[method.name]][[1]][[1]]
    
    CountYouden <- optimal.cutpoints(X = input$marker, status = input$event, tag.healthy = 0,
                                     methods = "Youden", data = df, pop.prev = NULL,
                                     control = control.cutpoints(), ci.fit = FALSE, conf.level = 0.95, trace = FALSE)
    
    pl <- ggplot(df, aes(x=biomarker)) +
      geom_histogram(fill="#2c3e50") +
      geom_vline(aes(xintercept=cutpoint))
    
    output$youdenTable <- renderTable( { rev.Youdenptable }, width = "50em", bordered = TRUE )
    output$youdenHist<- renderPlotly( { ggplotly( pl) })
    output$YoudenPlot <- renderPlot( { plot(CountYouden, which = 1) } )
    
  }
  display.results.outcome.roc <- function( input, output, session, df ) {
    
    biomarker <- df[, input$marker ]
    methods <- "ROC01"
    
    optimal.cutpoint.ROC <- optimal.cutpoints( X = input$marker, status = input$event, tag.healthy = 0,
                                               methods = methods, data = df, pop.prev = NULL,
                                               control = control.cutpoints(), ci.fit = FALSE, conf.level = 0.95, trace = FALSE )
    Youdenptable <- summary( optimal.cutpoint.ROC )$p.table$Global
    Youdenptable$AUC_CI <- NULL
    
    for(k in methods) {
      colnames(Youdenptable[[k]][[1]]) <- k  
    }
    
    rev.row.names <- attributes(Youdenptable)[[1]]
    rev.col.names <- c("method", "cutoff", "se", "sp", "PPV", "NPV", "DLR.Positive", "DLR.Negative", "FP", "FN")
    rev.Youdenptable <- matrix(ncol=10, nrow=length(methods), dimnames=list(rev.row.names, rev.col.names))
    for(k in 1:length(rev.row.names)) {
      method.name <- rev.row.names[[k]]
      rev.Youdenptable[k, 1] <- method.name
      for(j in 2:10){
        rev.Youdenptable[k, j] <- round(Youdenptable[[method.name]][[1]][[j - 1]], digits = 3)
      }
    }
    cutpoint <- Youdenptable[[method.name]][[1]][[1]]
    
    CountROC <- optimal.cutpoints(X = input$marker, status = input$event, tag.healthy = 0,
                                  methods = "ROC01", data = df, pop.prev = NULL,
                                  control = control.cutpoints(), ci.fit = FALSE, conf.level = 0.95, trace = FALSE)
    
    pl <- ggplot( df, aes(x=biomarker )) +
      geom_histogram(fill="#2c3e50") +
      geom_vline(aes(xintercept=cutpoint))
    
    
    output$ROCTable <- renderTable( {rev.Youdenptable}, width = "50em", bordered = TRUE )
    output$ROCHist<- renderPlotly( { ggplotly( pl) })
    output$ROCPlot <- renderPlot( { plot( CountROC, which = 1 ) } )
    
  }
  display.results.survival.cutp <- function( input, output, session, df ) {
    biomarker <- df[, input$marker ]
    vector.survival <- df[, input$survival ]
    vector.event <- df[, input$event ]
    
    cph1 <- coxph( Surv( vector.survival, vector.event ) ~ biomarker, data=df )
    result.frame <- data.frame( cutp( cph1 ) )
    cutpoint <- signif( result.frame[[ 1, 1 ]], digits = 4 )
    p.value <- signif( result.frame[[ 1, 4 ]], digits = 4 )
    
    length.lower <- length( biomarker[ biomarker < cutpoint ])
    length.upper <- length( biomarker[ biomarker > cutpoint ])
    length.lower.ratio <- length( biomarker[ biomarker < cutpoint ])/length( biomarker )
    length.upper.ratio <- length( biomarker[ biomarker > cutpoint ])/length( biomarker )
    
    category <- ifelse( biomarker < cutpoint, "low", "high" )
    category.df <- data.frame( df, category )
    
    
    l <- length(biomarker)
    x <- rep(NA, l)
    x[ which( biomarker < cutpoint)] <- 0
    x[ which( biomarker > cutpoint)] <- 1
    
    res.cox <- do.call(
      coxph,
      list( formula = Surv( vector.survival, vector.event ) ~ x, data = category.df)
    )
    
    model <- summary( res.cox )
    coef <- model$coefficients
    HR <- signif( coef[ 2 ], digits = 3 )
    q <- 1-(1-95/100)/2
    z <- qnorm( q )
    HR.lower <- signif( exp( coef[1] - z * coef[3] ), digits = 3 )
    HR.upper <- signif( exp( coef[1] + z * coef[3] ), digits = 3 )
    CI <- paste(" ( ", HR.lower, "-", HR.upper, " ) " )
    p <- signif( model$sctest[ "pvalue" ], digits = 3 )
    HR.total.table <- as.data.frame(cbind( HR,CI, p ))
    colnames(HR.total.table) <- c("HR", "CI", "P-value")
    
    result.table.row.names <- c( "result" )
    result.table.col.names <- c( "Cutpoint", "Biomarker < Cutpoint", "Biomarker > Cutpoint" )
    result.table <- matrix( ncol = 3, nrow = 1, dimnames = list( result.table.row.names, result.table.col.names ) )
    result.table[ 1, 1 ] <- cutpoint
    result.table[ 1, 2 ] <- paste( length.lower, "(", percent( length.lower.ratio ), ")")
    result.table[ 1, 3 ] <- paste( length.upper, "(", percent( length.upper.ratio ), ")")
    
    output$survivalCutpTable <- renderTable( result.table, width = "40em", bordered = TRUE  )
    
    fit <- do.call(
      survfit,
      list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df )
    )
    
    pl <- ggplot(df, aes(x=biomarker)) +
      geom_histogram(fill="#2c3e50") +
      geom_vline(aes(xintercept=cutpoint))
    
    ggfit <- ggsurv( fit, surv.col = c( "#2c3e50", "Red" ))
    
    
    output$survivalcutpHist<- renderPlotly( { ggplotly( pl) })
    output$HRcutpTable <- renderTable( HR.total.table, width = "40em", bordered = TRUE  )
    output$survivalCutpGraphSurv <- renderPlotly( { ggplotly( ggfit ) } )
    
  }
  
  display.results.survival.maxstat <- function( input, output, session, df ) {
    biomarker <- df[, input$marker ]
    vector.survival <- df[, input$survival ]
    vector.event <- df[, input$event ]
    
    
    mod <- maxstat.test( 
      Surv( vector.survival, vector.event ) ~ biomarker, 
      data=df, smethod="LogRank", pmethod="Lau92", iscores=TRUE
    )
    
    
    cutpoint <- signif(mod$estimate[[ 1 ]], digits = 4)
    p.value <- mod$p.value
    
    y <- mod$stats
    x <-mod$cuts
    dataxy <- data.frame(x, y)
    
    maxstatplot <-  ggplot( dataxy, aes( x=x, y=y ) ) + 
      geom_line(colour= "#2c3e50") + 
      geom_point(colour= "#2c3e50") +
      geom_vline(aes(xintercept=cutpoint)) +
      labs(x="biomarker",y="Standarized log-rank statistics") 
    
    length.lower <- length(biomarker[biomarker < cutpoint])
    length.upper <- length(biomarker[biomarker > cutpoint])
    length.lower.ratio <- length(biomarker[biomarker < cutpoint])/length(biomarker)
    length.upper.ratio <- length(biomarker[biomarker > cutpoint])/length(biomarker)
    
    category <- ifelse( biomarker < cutpoint, "low", "high" )
    category.df <- data.frame( df, category )
    
    l <- length(biomarker)
    x <- rep(NA, l)
    x[which(biomarker < cutpoint)] <- 0
    x[which(biomarker > cutpoint)] <- 1
    
    res.cox <- do.call(
      coxph,
      list( formula = Surv( vector.survival, vector.event ) ~ x, data = category.df)
    )
    
    model <- summary( res.cox )
    coef <- model$coefficients
    HR <- signif( coef[ 2 ], digits = 3)
    q <- 1-(1-95/100)/2
    z <- qnorm( q )
    HR.lower <- signif( exp(coef[ 1 ] - z * coef[ 3 ] ), digits = 3 )
    HR.upper <- signif( exp( coef[ 1 ] + z * coef[ 3 ] ), digits = 3 )
    CI <- paste( " ( ", HR.lower, "-", HR.upper, " ) " )
    p <- signif( model$sctest[ "pvalue" ], digits = 3 )
    HR.total.table <- as.data.frame( cbind( HR,CI, p ) )
    colnames( HR.total.table ) <- c( "HR", "CI", "P-value" )
    
    result.table.row.names <- c( "result" )
    result.table.col.names <- c( "Cutpoint", "Biomarker < Cutpoint", "Biomarker > Cutpoint" )
    result.table <- matrix( ncol = 3, nrow = 1, dimnames = list( result.table.row.names, result.table.col.names ) )
    result.table[ 1, 1 ] <- cutpoint
    result.table[ 1, 2 ] <- paste( length.lower, "(", percent( length.lower.ratio ), ") " )
    result.table[ 1, 3 ] <- paste( length.upper, "(", percent( length.upper.ratio ), ") " )
    
    output$survivalMaxstatTable <- renderTable( result.table, width = "40em", bordered = TRUE )
    output$survivalMaxstatGraphResult <- renderPlotly( { ggplotly( maxstatplot) } )
    
    
    fit <- do.call(
      survfit,
      list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df )
    )
    
    pl <- ggplot( df, aes( x=biomarker )) +
      geom_histogram( fill="#2c3e50" ) +
      geom_vline( aes( xintercept=cutpoint ))
    
    ggfit <- ggsurv( fit, surv.col = c( "#2c3e50", "Red" ))
    
    
    
    output$survivalMaxstatHist<- renderPlotly( { ggplotly( pl) })
    output$HRMaxstatTable <- renderTable( HR.total.table, width = "40em", bordered = TRUE )
    output$survivalMaxstatGraphSurv <- renderPlotly( { ggplotly(ggfit) } )
    
  }
  display.results.adapt <- function( input, output, session, df ) {
    biomarker <- df[, input$marker ]
    vector.survival <- df[, input$survival ]
    vector.event <- df[, input$event ]
    
    res.cox <- do.call(
      coxph,
      list( formula = Surv( vector.survival, vector.event ) ~ biomarker, data = df)
    )
    
    
    
    cutpres <- data.frame( cutp( res.cox ) )
    cutpres <- data.frame( cutpres$biomarker.biomarker, cutpres$biomarker.p )
    cutpres <- cutpres[ order( cutpres$cutpres.biomarker.biomarker ), ]
    cutpres.bind <- cbind( cutpres$cutpres.biomarker.p, cutpres$cutpres.biomarker.p )
    rownames( cutpres.bind ) <- cutpres$cutpres.biomarker.biomarker
    rev.cutpres <- data.frame( t( cutpres.bind ))
    colnames( rev.cutpres ) <- rownames( cutpres.bind )
    rownames( rev.cutpres ) <- c( ""," ")
    y.axisSettings <- list(
      title = "",
      zeroline = FALSE,
      showline = FALSE,
      showticklabels = FALSE,
      showgrid = FALSE,
      ticks = ""
    )
    x.axisSettings <- list(
      title = "Biomarker"
    )
    
    output$heatmap <- renderPlotly( plot_ly( z = data.matrix(rev.cutpres),
                                             x = colnames(rev.cutpres),
                                             y = rownames(rev.cutpres),
                                             type = "heatmap",
                                             colorbar = list(xanchor = "left", yanchor = "top", ypad = 0, xpad = 0, lenmode = "pixels", len= 150, nticks=3))
                                    %>%
                                      layout(yaxis = y.axisSettings, xaxis = x.axisSettings)
    )
    
    output$click <- renderText({
      eventClick <- event_data( "plotly_click" )
      val <- eventClick$x
      if (is.null(val)) "Click on the heatmap to generate plots" else NULL
    })
    
  }
  display.results.adaptObserve <- observe({
    eventClick <- event_data( "plotly_click" )
    val <- eventClick$x
    df <- filedata()
    
    if( is.null( val ) || is.null( df ) || input$divisonBase != "adapt" || input$nrGroups != 2  ) {
      return( NULL )
    } else {
      biomarker <- df[, input$marker ]
      vector.survival <- df[, input$survival ]
      vector.event <- df[, input$event ]
      
      
      category <- ifelse( biomarker < val, "low", "high" )
      category.df <- data.frame( df, category )
      
      length.lower <- length( biomarker[ biomarker < val ])
      length.upper <- length( biomarker[ biomarker > val ])
      length.lower.ratio <- length( biomarker[ biomarker < val ])/length( biomarker )
      length.upper.ratio <- length( biomarker[ biomarker > val ])/length( biomarker )
      
      
      fit <- do.call(
        survfit,
        list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df )
      )
      
      l <- length( biomarker )
      x <- rep( NA, l )
      x[ which( biomarker < val )] <- 0
      x[ which( biomarker > val )] <- 1
      
      res.cox <- do.call(
        coxph,
        list( formula = Surv( vector.survival, vector.event ) ~ x, data = category.df)
      )
      
      model <- summary( res.cox )
      coef <- model$coefficients
      HR <- signif(coef[ 2 ], digits = 3)
      q <- 1-(1-95/100)/2
      z <- qnorm( q )
      HR.lower <- signif( exp( coef[ 1 ] - z * coef[ 3 ]), digits = 3 )
      HR.upper <- signif( exp( coef[ 1 ] + z * coef[ 3 ]), digits = 3)
      CI <- paste( " ( ", HR.lower, "-", HR.upper, " ) " )
      p <- signif(model$sctest[ "pvalue" ], digits = 3 )
      HR.total.table <- as.data.frame( cbind( HR,CI, p ) )
      colnames( HR.total.table ) <- c( "HR", "CI", "P-value" )
      
      result.table.row.names <- c( "result" )
      result.table.col.names <- c( "Cutpoint", "Biomarker < Cutpoint", "Biomarker > Cutpoint" )
      result.table <- matrix( ncol = 3, nrow = 1, dimnames = list( result.table.row.names, result.table.col.names) )
      result.table[1, 1] <- signif( val, digits = 4 )
      result.table[1, 2] <- paste( length.lower, "(", percent( length.lower.ratio ), ") " )
      result.table[1, 3] <- paste( length.upper, "(", percent( length.upper.ratio ), ") " )
      
      pl <- ggplot(df, aes(x=biomarker)) +
        geom_histogram(fill="#2c3e50") +
        geom_vline(aes(xintercept=val))
      
      ggfit <- ggsurv( fit, surv.col = c( "#2c3e50", "Red" ) )
      
      
      output$adaptresultTable <- renderTable( result.table, width = "40em", bordered = TRUE )
      output$adaptHist <- renderPlotly( { ggplotly( pl) })
      output$adaptHRTable <- renderTable( HR.total.table, width = "40em", bordered = TRUE )
      output$adaptResult <- renderPlotly( { ggplotly(ggfit) } ) 
      
    }
  })
  
  display.results.threeGroups <- function( input, output, session, df ) {
    
    func <- function(df) {
      df <- df[, c(input$marker, input$survival, input$event)]
      df <- na.omit(df)
      biomarker <- df[, input$marker ]
      vector.survival <- df[, input$survival ]
      vector.event <- df[, input$event ]
      res <- rhier( times = vector.survival, status = vector.event, x = biomarker, ns = 15, alt = 'increase' )
      
    }
    
    rolr <- func( df )
    low.cutoff = rolr[[ 1 ]][[ 1 ]]
    high.cutoff = rolr[[ 1 ]][[ 2 ]]
    biomarker <- df[, input$marker ]
    vector.survival <- df[, input$survival ]
    vector.event <- df[, input$event ]
    
    
    length.lower <- length( biomarker[ biomarker < low.cutoff ])
    length.upper <- length( biomarker[ biomarker > high.cutoff ])
    length.medium <- length( biomarker ) - length.lower - length.upper
    length.lower.ratio <- length.lower / length( biomarker )
    length.upper.ratio <- length.upper / length( biomarker )
    length.medium.ratio <- length.medium / length( biomarker )
    
    
    split.table.row.names <- c( "result" )
    split.table.col.names <- c( "Cutpoint 1", "Cutpoint 2", "Low", "Medium", "High" )
    split.table <- matrix( ncol = 5, nrow = 1, dimnames = list( split.table.row.names, split.table.col.names))
    split.table[ 1, 1 ] <- low.cutoff
    split.table[ 1, 2 ] <- high.cutoff
    split.table[ 1, 3 ] <- paste( length.lower, "(", percent( length.lower.ratio ), ") " )
    split.table[ 1, 4 ] <- paste( length.medium, "(", percent( length.medium.ratio ), ") " )
    split.table[ 1, 5 ] <- paste( length.upper, "(", percent( length.upper.ratio ), ") " )
    
    
    category <- ifelse( biomarker < low.cutoff, "low", ifelse( biomarker > high.cutoff, "high", "medium" ))
    category.df <- data.frame( biomarker, vector.survival, vector.event, category )
    
    category.df.lowmed <- data.frame( subset( category.df, subset= category == "medium" | category == "low" ))
    category.df.lowhigh <- data.frame( subset( category.df, subset= category == "high" | category == "low" ))
    category.df.medhigh <- data.frame( subset( category.df, subset= category == "high" | category == "medium" ))
    
    
    fit <- do.call(
      survfit,
      list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df )
    )
    
    if (low.cutoff != high.cutoff) {
      
      lowhigh <- do.call(
        survfit,
        list( formula = Surv(  vector.survival, vector.event ) ~ category, data = category.df.lowhigh)
      )
      
      lowmedium <- do.call(
        survfit,
        list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df.lowmed)
      )
      
      mediumhigh <- do.call(
        survfit,
        list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df.medhigh)
      )
      
      cat.df <- data.frame( biomarker, vector.survival, vector.event )
      
      categorize.low <- subset( cat.df, subset = biomarker < low.cutoff | biomarker > high.cutoff )
      category.low = ifelse( categorize.low$biomarker < low.cutoff, "low", "high" )
      cat.low <- data.frame( categorize.low, category.low )
      
      lowhigh.surv <- do.call(
        coxph,
        list( formula = Surv( vector.survival, vector.event ) ~ category.low, data = cat.low)
      )
      
      categorize.med <- subset( cat.df, subset = biomarker < high.cutoff )
      category.med = ifelse( categorize.med$biomarker < low.cutoff, "medium", "high" )
      cat.med <- data.frame(categorize.med, category.med)
      
      lowmedium.surv <- do.call(
        coxph,
        list( formula = Surv( vector.survival, vector.event ) ~ category.med, data = cat.med)
      )
      
      categorize.high <- subset( cat.df, subset = biomarker > low.cutoff )
      category.high = ifelse( categorize.high$biomarker < high.cutoff, "medium", "high" )
      cat.high <- data.frame( categorize.high, category.high )
      
      mediumhigh.surv <- do.call(
        coxph,
        list( formula = Surv( vector.survival, vector.event ) ~ category.high, data = cat.high)
      )
      
      model <- summary( lowhigh.surv )
      coef <- model$coefficients
      group <- "Low vs High"
      HR <- signif(coef[ 2 ], digits = 3 )
      q <- 1-(1-95/100)/2
      z <- qnorm( q )
      HR.lower <- signif( exp( coef[ 1 ] - z * coef[ 3 ]), digits = 3 )
      HR.upper <- signif( exp( coef[ 1 ] + z * coef[ 3 ]), digits = 3 )
      CI <- paste( " (", HR.lower, "-", HR.upper, ") " )
      p <- signif( model$sctest[ "pvalue" ], digits = 3 )
      HR.low.table <- as.data.frame( cbind( group,HR,CI, p ) )
      colnames( HR.low.table ) <- c( "Group", "HR", "CI", "P-value" )
      
      model.medium <- summary( lowmedium.surv )
      c.medium <- model.medium$coefficients
      group.medium <- "Low vs Medium"
      HR.medium <- signif( c.medium[ 2 ], digits = 3 )
      q.medium <- 1-(1-95/100)/2
      z.medium <- qnorm( q.medium )
      HR.lower.medium <- signif( exp( c.medium[ 1 ] - z * c.medium[ 3 ]), digits = 3 )
      HR.upper.medium <- signif( exp( c.medium[ 1 ] + z * c.medium[ 3 ]), digits = 3 )
      CI.medium <- paste( " ( ", HR.lower.medium, "-", HR.upper.medium, " ) " )
      p.medium <- signif( model.medium$sctest[ "pvalue" ], digits = 3 )
      HR.medium.table <- as.data.frame( cbind( group.medium, HR.medium, CI.medium, p.medium ))
      colnames( HR.medium.table ) <- c( "Group", "HR", "CI", "P-value" )
      
      model.high <- summary( mediumhigh.surv )
      c.high <- model.high$coefficients
      group.high <- "Medium vs High"
      HR.high <- signif( c.high[ 2 ], digits = 3 )
      q.high <- 1-(1-95/100)/2
      z.high <- qnorm( q.high )
      HR.lower.high <- signif( exp( c.high[ 1 ] - z * c.high[ 3 ]), digits = 3 )
      HR.upper.high <- signif( exp( c.high[ 1 ] + z * c.high[ 3 ]), digits = 3 )
      CI.high <- paste( " (", HR.lower.high, "-", HR.upper.high, ") " )
      p.high <- signif( model.high$sctest[ "pvalue" ], digits = 3 ) 
      HR.high.table <- as.data.frame( cbind( group.high, HR.high, CI.high, p.high))
      colnames( HR.high.table ) <- c( "Group", "HR", "CI", "P-value" )
      
      
      HR.total.table <- rbind( HR.low.table, HR.medium.table, HR.high.table )
      
      
      lowmedium.fit <- GGally::ggsurv( lowmedium, surv.col = c( "Red", "Blue" ))
      lowhigh.fit<- GGally::ggsurv( lowhigh, surv.col = c( "#2c3e50", "Red" ))
      mediumhigh.fit <- GGally::ggsurv( mediumhigh, surv.col = c( "#2c3e50", "Blue" ))
      
      output$lowmediumfit <- renderPlotly( { ggplotly(lowmedium.fit) })
      output$lowhighfit <- renderPlotly( { ggplotly( lowhigh.fit) })
      output$mediumhighfit <- renderPlotly( { ggplotly(mediumhigh.fit) })
      output$Check <- renderTable( HR.total.table, bordered = TRUE, width = "40em" )
      
    } else { 
      cat.df <- data.frame(biomarker, vector.survival, vector.event)
      
      categorize.low <- subset( cat.df, subset = biomarker < low.cutoff | biomarker > high.cutoff )
      category.low = ifelse( categorize.low$biomarker < low.cutoff, "low", "high" )
      cat.low <- data.frame( categorize.low, category.low )
      
      lowhigh.surv <- do.call(
        coxph,
        list( formula = Surv( vector.survival, vector.event ) ~ category.low, data = cat.low)
      )
      
      
      model <- summary( lowhigh.surv )
      coef <- model$coefficients
      group <- "Low vs High"
      HR <- signif(coef[ 2 ], digits = 3 )
      q <- 1-(1-95/100)/2
      z <- qnorm( q )
      HR.lower <- signif( exp( coefrow[ 1 ] - z * coefrow[ 3 ]), digits = 3 )
      HR.upper <- signif( exp(coefrow[ 1 ] + z * coefrow[ 3 ] ), digits = 3 )
      CI <- paste( " (", HR.lower, "-", HR.upper, ") " )
      p <- signif( model$sctest[ "pvalue" ], digits = 3 )
      HR.low.table <- as.data.frame( cbind( group, HR, CI, p))
      colnames(HR.low.table) <- c( "Group", "HR", "CI", "P-value" )
      
      output$lowCheck <- renderTable( HR.low.table, bordered = TRUE, width = "40em" )
    }
    
    output$threeSplits <- renderTable( split.table, bordered = TRUE, width = "40em" )
    
    pl <- ggplot(df, aes(x=biomarker)) +
      geom_histogram(fill="#2c3e50") +
      geom_vline(aes(xintercept=low.cutoff)) + 
      geom_vline(aes(xintercept=high.cutoff))
    
    ggfit <- GGally::ggsurv(fit, surv.col = c( "#2c3e50", "Red", "Blue" ))
    
    
    output$rolrHist<- renderPlotly( { ggplotly( pl) })
    output$threeGraph <- renderPlotly( { ggplotly( ggfit ) })
    
  }
  
  display.results.threeAdopt <- function( input, output, session, df ) {
    marker <- df[[ input$marker ]]
    marker.min <- min(marker)
    marker.max <- max(marker)
    
    
    output$tadaptSliderMin <- renderUI( 
      sliderInput( "tadaptSliderMin",
                   "Choose min cutoff value:", 
                   marker.min, 
                   marker.max, 
                   (marker.min + marker.max) / 2, 
                   step = 0.01,
                   width = "40em"
      )
    )
    output$tadaptSliderMax <- renderUI( 
      sliderInput( "tadaptSliderMax",
                   "Choose max cutoff value:", 
                   marker.min, 
                   marker.max, 
                   (marker.min + marker.max) / 2, 
                   step = 0.01,
                   width = "40em"
      )
    )
  }
  
  display.results.tadaptObserve <- observe({
    
    df <- filedata()
    
    if( is.null( input$tadaptSliderMin ) || is.null( input$tadaptSliderMax ) || is.null( df ) || input$tdivisonBase != "adapt" || input$nrGroups != 3  ) {
      return( NULL )
    } else {
      val <- c( input$tadaptSliderMin, input$tadaptSliderMax )
      val.min <- min( val )
      val.max <- max( val )
      
      biomarker <- df[, input$marker ]
      vector.survival <- df[, input$survival ]
      vector.event <- df[, input$event ]
      
      category = ifelse( biomarker < val.min, "low", ifelse( biomarker > val.max, "high", "medium" ))
      category.df <- data.frame( biomarker, vector.survival, vector.event, category)
      category.df.lowmed <- subset( category.df, subset= category == "low" | category == "medium" )
      category.df.lowhigh <- subset( category.df, subset= category == "low" | category == "high" )
      category.df.medhigh <- subset( category.df, subset= category == "high" | category == "medium" )
      
      fit <- do.call(
        survfit,
        list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df )
      )
      
      if (val.min != val.max) {
        
        cat.df <- data.frame( biomarker, vector.survival, vector.event )
        
        categorize.low <- subset( cat.df, subset = biomarker < val.min | biomarker > val.max)
        category.low = ifelse( categorize.low$biomarker < val.min, "low", "high" )
        cat.low <- data.frame( categorize.low, category.low )
        
        lowhigh.surv <- do.call(
          coxph,
          list( formula = Surv( vector.survival, vector.event ) ~ category.low, data = cat.low)
        )
        
        categorize.med <- subset( cat.df, subset = biomarker < val.max )
        category.med = ifelse( categorize.med$biomarker < val.min, "low", "medium" )
        cat.med <- data.frame( categorize.med, category.med )
        
        lowmedium.surv <- do.call(
          coxph,
          list( formula = Surv( vector.survival, vector.event ) ~ category.med, data = cat.med )
        )
        
        categorize.high <- subset( cat.df, subset = biomarker > val.min )
        category.high = ifelse( categorize.high$biomarker < val.max, "medium", "high" )
        cat.high <- data.frame( categorize.high, category.high )
        
        mediumhigh.surv <- do.call(
          coxph,
          list( formula = Surv( vector.survival, vector.event ) ~ category.high, data = cat.high)
        )
        
        model <- summary( lowhigh.surv )
        coef <- model$coefficients
        group <- "Low vs High"
        HR <- signif( coef[ 2 ], digits = 3 )
        q <- 1-(1-95/100)/2
        z <- qnorm( q )
        HR.lower <- signif(exp(coef[ 1 ] - z * coef[ 3 ]), digits = 3)
        HR.upper <- signif(exp(coef[ 1 ] + z * coef[ 3 ]), digits = 3)
        CI <- paste("(", HR.lower, "-", HR.upper, ")")
        p <- signif(model$sctest[ "pvalue" ], digits = 3 )
        HR.low.table <- as.data.frame( cbind( group, HR, CI, p ) )
        colnames( HR.low.table ) <- c( "Group", "HR", "CI", "P-value" )
        
        model.medium <- summary( lowmedium.surv )
        c.medium <- model.medium$coefficients
        group.medium <- "Low vs Medium"
        HR.medium <- signif(c.medium[ 2 ], digits = 3 )
        q.medium <- 1-(1-95/100)/2
        z.medium <- qnorm( q.medium )
        HR.lower.medium <- signif( exp( c.medium[ 1 ] - z * c.medium[ 3 ]), digits = 3 )
        HR.upper.medium <- signif( exp( c.medium[ 1 ] + z * c.medium[ 3 ]), digits = 3 )
        CI.medium <- paste( " (", HR.lower.medium, "-", HR.upper.medium, ") " )
        p.medium <- signif( model.medium$sctest[ "pvalue" ], digits = 3 )
        HR.medium.table <- as.data.frame( cbind( group.medium, HR.medium, CI.medium, p.medium))
        colnames( HR.medium.table ) <- c("Group", "HR", "CI", "P-value")
        
        model.high <- summary( mediumhigh.surv )
        c.high <- model.high$coefficients
        group.high <- "Medium vs High"
        HR.high <- signif(c.high[ 2 ], digits = 3)
        q.high <- 1-(1-95/100)/2
        z.high <- qnorm( q.high )
        HR.lower.high <- signif( exp( c.high[ 1 ] - z * c.high[ 3 ]), digits = 3 )
        HR.upper.high <- signif( exp( c.high[ 1 ] + z * c.high[ 3 ]), digits = 3 )
        CI.high <- paste( " (", HR.lower.high, "-", HR.upper.high, ") " )
        p.high <- signif( model.high$sctest[ "pvalue" ], digits = 3 )
        HR.high.table <- as.data.frame( cbind( group.high, HR.high, CI.high, p.high))
        colnames( HR.high.table ) <- c("Group", "HR", "CI", "P-value")
        HR.total.table <- rbind( HR.low.table, HR.medium.table, HR.high.table)
        
        lowhigh <- do.call(
          survfit,
          list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df.lowhigh)
        )
        
        lowmedium <- do.call(
          survfit,
          list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df.lowmed)
        )
        
        mediumhigh <- do.call(
          survfit,
          list( formula = Surv( vector.survival, vector.event ) ~ category, data = category.df.medhigh)
        )
        
        
        lowmedium.fit <- GGally::ggsurv( lowmedium, surv.col = c( "Red", "Blue" ))
        lowhigh.fit<- GGally::ggsurv( lowhigh, surv.col = c( "#2c3e50", "Red" ))
        mediumhigh.fit <- GGally::ggsurv( mediumhigh, surv.col = c( "#2c3e50", "Blue" ))
        
        output$lowmediumadapt <- renderPlotly( { ggplotly(lowmedium.fit) } )
        output$lowhighadapt <- renderPlotly( { ggplotly(lowhigh.fit) } )
        output$mediumhighadapt <- renderPlotly( { ggplotly(mediumhigh.fit) } )
        output$Checkadapt <- renderTable(HR.total.table, bordered = TRUE, width = "40em" )
        
      } else {   NULL        }
      
      pl <- ggplot(df, aes(x=biomarker)) +
        geom_histogram(fill="#2c3e50") +
        geom_vline(aes(xintercept=val.min)) + 
        geom_vline(aes(xintercept=val.max))
      
      ggfit <- GGally::ggsurv(fit, surv.col = c("#2c3e50", "Red", if (val.min != val.max) {"Blue"} else {NULL}))
      
      
      length.lower <- length( biomarker[ biomarker < val.min ] )
      length.upper <- length( biomarker[ biomarker > val.max ])
      length.medium <- length( biomarker ) - length.lower - length.upper
      length.lower.ratio <- length.lower / length( biomarker )
      length.upper.ratio <- length.upper / length( biomarker )
      length.medium.ratio <- length.medium / length( biomarker )
      
      
      split.table.row.names <- c( "result" )
      split.table.col.names <- c( "Cutpoint 1", "Cutpoint 2", "Low", "Medium", "High" )
      split.table <- matrix( ncol = 5, nrow = 1, dimnames = list( split.table.row.names, split.table.col.names ))
      split.table[1, 1] <- val.min
      split.table[1, 2] <- val.max
      split.table[1, 3] <- paste(length.lower, "(", percent(length.lower.ratio), ")")
      split.table[1, 4] <- paste(length.medium, "(", percent(length.medium.ratio), ")")
      split.table[1, 5] <- paste(length.upper, "(", percent(length.upper.ratio), ")")
      
      output$threeadaptTable <- renderTable( split.table, bordered = TRUE, width = "40em")
      output$tadaptHist<- renderPlotly( { ggplotly( pl) })
      output$tadaptResult <- renderPlotly( { ggplotly( ggfit) } )
    }
  })
})